# -*- coding: utf-8 -*-
# Generated by Django 1.9.9 on 2016-11-20 12:04
from __future__ import unicode_literals

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion
import django_countries.fields


class Migration(migrations.Migration):

    dependencies = [
        ('leprikon', '0015_protect_leader'),
    ]

    operations = []
    for model_name in ['participant', 'clubregistration', 'eventregistration']:
        operations += [
            migrations.AlterField(
                model_name=model_name,
                name='citizenship',
                field=django_countries.fields.CountryField(max_length=2, verbose_name='citizenship'),
            ),
        ]



# -*- coding: utf-8 -*-
# Generated by Django 1.9.9 on 2016-08-26 09:10
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion
import leprikon.models.fields


def move_discounts(apps, schema_editor):

    ClubRegistration = apps.get_model('leprikon', 'ClubRegistration')
    ClubRegistrationDiscount = apps.get_model('leprikon', 'ClubRegistrationDiscount')

    for reg in ClubRegistration.objects.exclude(discount=0):
        discount = ClubRegistrationDiscount()
        discount.registration = reg
        if reg.canceled:
            discount.period = reg.club.periods.filter(end__gt=reg.created).last()
        else:
            discount.period = reg.club.periods.filter(end__gt=reg.created).first()
        discount.discount = reg.discount
        discount.explanation = reg.explanation
        discount.save()


class Migration(migrations.Migration):

    dependencies = [
        ('leprikon', '0009_cmsplugin_ptr'),
    ]

    operations = [
        migrations.CreateModel(
            name='ClubRegistrationDiscount',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('discount', leprikon.models.fields.PriceField(decimal_places=0, default=0, max_digits=10, verbose_name='discount')),
                ('explanation', models.CharField(blank=True, default='', max_length=250, verbose_name='discount explanation')),
                ('period', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='discounts', to='leprikon.ClubPeriod', verbose_name='period')),
                ('registration', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='discounts', to='leprikon.ClubRegistration', verbose_name='registration')),
            ],
            options={
                'ordering': ('period',),
                'verbose_name': 'club discount',
                'verbose_name_plural': 'club discounts',
            },
        ),
        migrations.AlterUniqueTogether(
            name='clubregistrationdiscount',
            unique_together=set([('registration', 'period')]),
        ),
        migrations.RunPython(move_discounts),
        migrations.RemoveField(
            model_name='clubregistration',
            name='discount',
        ),
        migrations.RemoveField(
            model_name='clubregistration',
            name='explanation',
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.9.9 on 2017-01-28 00:24
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion
import filer.fields.file


class Migration(migrations.Migration):

    dependencies = [
        ('filer', '0006_auto_20160623_1627'),
        ('leprikon', '0025_club_type'),
    ]

    operations = [
        migrations.CreateModel(
            name='PrintSetup',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=150, verbose_name='name')),
                ('top', models.IntegerField(blank=True, default=20, verbose_name='margin top')),
                ('left', models.IntegerField(blank=True, default=20, verbose_name='margin left')),
                ('right', models.IntegerField(blank=True, default=20, verbose_name='margin right')),
                ('bottom', models.IntegerField(blank=True, default=20, verbose_name='margin bottom')),
                ('template', filer.fields.file.FilerFileField(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='+', to='filer.File', verbose_name='pdf template')),
            ],
            options={
                'ordering': ('name',),
                'verbose_name': 'print setup',
                'verbose_name_plural': 'print setups',
            },
        ),
        migrations.AddField(
            model_name='club',
            name='reg_printsetup',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='+', to='leprikon.PrintSetup', verbose_name='registration print setup'),
        ),
        migrations.AddField(
            model_name='clubtype',
            name='reg_printsetup',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='+', to='leprikon.PrintSetup', verbose_name='registration print setup'),
        ),
        migrations.AddField(
            model_name='event',
            name='reg_printsetup',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='+', to='leprikon.PrintSetup', verbose_name='registration print setup'),
        ),
        migrations.AddField(
            model_name='eventtype',
            name='reg_printsetup',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='+', to='leprikon.PrintSetup', verbose_name='registration print setup'),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.9.13 on 2018-01-16 13:51
from __future__ import unicode_literals

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('leprikon', '0008_leprikonsite_company_name'),
    ]

    operations = [
        migrations.AddField(
            model_name='subjectpayment',
            name='received_by',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.PROTECT, related_name='+', to=settings.AUTH_USER_MODEL, verbose_name='received by'),
        ),
    ]



from __future__ import unicode_literals

from django import forms
from django.core.exceptions import ValidationError
from django.forms.fields import CharField, EmailField, MultiValueField
from django.utils.translation import ugettext_lazy as _

from .fieldsetup import VerifiedEmailFieldSetup, fieldsetups
from .utils import get_code, send_code
from .widgets import VerifiedEmailWidget


class SendForm(forms.Form):
    email = EmailField()


class VerificationCodeField(CharField):
    default_error_messages = {
        'invalid': _('Enter a valid verification code.'),
    }

    def __init__(self, length, **kwargs):
        kwargs['max_length'] = kwargs['min_length'] = length
        kwargs['required'] = False
        super(VerificationCodeField, self).__init__(**kwargs)


class VerifiedEmailField(MultiValueField):

    def __init__(self, required=True, fieldsetup_id=None, max_length=None,
                 email_label=_('e-mail'), send_label=_('send verification code'), code_label=_('verification code'),
                 **kwargs):
        self.fieldsetup_id = fieldsetup_id or str(hash(self))
        self.fieldsetup = fieldsetups.setdefault(self.fieldsetup_id, VerifiedEmailFieldSetup(**kwargs))
        self.widget = VerifiedEmailWidget(
            send_label=send_label,
            fieldsetup_id=self.fieldsetup_id,
            email_attrs={'placeholder': email_label},
            code_attrs={'placeholder': code_label},
        )
        super(VerifiedEmailField, self).__init__((
            EmailField(label=email_label, required=required),
            VerificationCodeField(label=code_label, length=self.fieldsetup.code_length),
        ), require_all_fields=False, **kwargs)

    def clean(self, value):
        email = self.fields[0].clean(value[0])
        code = self.fields[1].clean(value[1])
        valid_code = get_code(email, self.fieldsetup)
        if email:
            if not valid_code:
                send_code(email, self.fieldsetup)
                raise ValidationError(_('The verification code has been sent to your email.'))
            else:
                if not code:
                    raise ValidationError(_('Enter verification code.'))
                elif code != valid_code:
                    raise ValidationError(_("The verification code doesn't match."))
        return email



# -*- coding: utf-8 -*-
# Generated by Django 1.9.9 on 2016-10-27 16:17
from __future__ import unicode_literals

from django.db import migrations
import leprikon.models.fields


class Migration(migrations.Migration):

    dependencies = [
        ('leprikon', '0012_place_on_delete_set_null'),
    ]

    operations = [
        migrations.AlterField(
            model_name='participant',
            name='birth_num',
            field=leprikon.models.fields.BirthNumberField(verbose_name='birth number'),
        ),
        migrations.AlterUniqueTogether(
            name='participant',
            unique_together=set([('user', 'birth_num')]),
        ),
    ]



from __future__ import unicode_literals

from datetime import timedelta
from random import randint

from django.core.cache import cache
from django.core.mail import EmailMultiAlternatives
from django.template.loader import get_template
from django.utils.timezone import now


def get_code(email, fieldsetup):
    try:
        expiration_time, code = cache.get(fieldsetup.cache_prefix + email)
    except:
        return None
    return code if expiration_time >= now() else None


def send_code(email, fieldsetup):
    # create code and expiration time
    context = dict(fieldsetup.mail_context)
    context['code'] = (get_code(email, fieldsetup) or
                       str(randint(10 ** (fieldsetup.code_length - 1), 10 ** fieldsetup.code_length - 1)))
    context['expiration_time'] = now() + timedelta(0, fieldsetup.code_ttl)
    # store code and expiration time in cache
    cache.set(fieldsetup.cache_prefix + email, (context['expiration_time'], context['code']))
    # create message
    msg = EmailMultiAlternatives(
        subject=fieldsetup.mail_subject,
        body=get_template(fieldsetup.mail_template_txt).render(context),
        from_email=fieldsetup.mail_from,
        to=[email],
        headers={'X-Mailer': fieldsetup.mail_mailer},
    )
    msg.attach_alternative(get_template(fieldsetup.mail_template_html).render(context), 'text/html')
    msg.send()



# -*- coding: utf-8 -*-
# Generated by Django 1.10.8 on 2017-10-31 22:56
from __future__ import unicode_literals

from django.db import migrations, models

from ..models.utils import generate_variable_symbol


def generate_variable_symbols(apps, schema_editor):
    SubjectRegistration = apps.get_model('leprikon', 'SubjectRegistration')
    for registration in SubjectRegistration.objects.all():
        registration.variable_symbol = generate_variable_symbol(registration)
        registration.save()


class Migration(migrations.Migration):

    dependencies = [
        ('leprikon', '0004_multiple_registrations'),
    ]

    operations = [
        migrations.AddField(
            model_name='subjectregistration',
            name='variable_symbol',
            field=models.IntegerField(db_index=True, editable=False, null=True, verbose_name='variable symbol'),
        ),
        migrations.RunPython(generate_variable_symbols),
    ]



from __future__ import unicode_literals

from django.forms.widgets import CheckboxSelectMultiple, RadioSelect
from django.utils.safestring import mark_safe


class BootstrapRenderer:

    def render_option(self, value, label, i):
        widget = self.choice_input_class(self.name, self.value, self.attrs.copy(), (value, label), i)
        row    = (
        )
        return row.format(
            tag     = widget.tag(),
            id      = widget.attrs['id'],
            label   = widget.choice_label,
        )

    def render(self):
        rows    = []
        i       = 0
        for value, label in self.choices:
            if isinstance(label, (list, tuple)):
                for v, l in label:
                    rows.append(self.render_option(v, l, i))
                    i += 1
            else:
                rows.append(self.render_option(value, label, i))
            i += 1
        return mark_safe('\n'.join(rows))


class RadioSelectBootstrap(RadioSelect):
    template_name = 'leprikon/widgets/multiple_input.html'
    option_template_name = 'leprikon/widgets/input_option.html'


class CheckboxSelectMultipleBootstrap(CheckboxSelectMultiple):
    template_name = 'leprikon/widgets/multiple_input.html'
    option_template_name = 'leprikon/widgets/input_option.html'



# -*- coding: utf-8 -*-
# Generated by Django 1.9.13 on 2018-03-18 13:42
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('leprikon', '0009_subjectpayment_received_by'),
    ]

    operations = [
        migrations.AddField(
            model_name='subject',
            name='code',
            field=models.PositiveSmallIntegerField(blank=True, default=0, verbose_name='accounting code'),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.12 on 2018-08-29 16:34
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('leprikon', '0017_citizenship'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='subjectregistration',
            name='participant_citizenship_old',
        ),
        migrations.RemoveField(
            model_name='participant',
            name='citizenship_old',
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.9.9 on 2016-11-28 12:19
from __future__ import unicode_literals

from django.db import migrations, models


def migrate_entries(apps, schema_editor):
    ClubJournalEntry = apps.get_model('leprikon', 'clubjournalentry')
    ClubRegistration = apps.get_model('leprikon', 'clubregistration')

    for entry in ClubJournalEntry.objects.all():
        entry.registrations = ClubRegistration.objects.filter(club=entry.club, participant_birth_num__in=entry.participants.values_list('birth_num', flat=True))


class Migration(migrations.Migration):

    dependencies = [
        ('leprikon', '0020_registrations_cleanup'),
    ]

    operations = [
        migrations.AddField(
            model_name='clubjournalentry',
            name='registrations',
            field=models.ManyToManyField(blank=True, related_name='journal_entries', to='leprikon.ClubRegistration', verbose_name='participants'),
        ),
        migrations.RunPython(migrate_entries),
        migrations.RemoveField(
            model_name='clubjournalentry',
            name='participants',
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.9.13 on 2018-04-02 19:42
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('leprikon', '0010_subject_code'),
    ]

    operations = [
        migrations.AddField(
            model_name='subjectregistration',
            name='payment_requested',
            field=models.DateTimeField(editable=False, null=True, verbose_name='payment request time'),
        ),
    ]



from __future__ import unicode_literals

from django.contrib import admin
from django.utils.translation import ugettext_lazy as _

from ..models.courses import Course
from ..models.events import Event
from ..models.roles import Leader
from ..models.schoolyear import SchoolYear
from ..models.subjects import Subject, SubjectGroup, SubjectType


class SchoolYearListFilter(admin.FieldListFilter):
    def __init__(self, field, request, params, model, model_admin, field_path):
        try:
            request.school_year = SchoolYear.objects.get(year=params['year'])
        except:
            pass
        self.school_year = request.school_year
        super(SchoolYearListFilter, self).__init__(field, request, params, model, model_admin, field_path)

    def expected_parameters(self):
        return ['year']

    def choices(self, cl):
        return [
            {
                'selected': school_year == self.school_year,
                'query_string': cl.get_query_string({'year': school_year.year}),
                'display': school_year,
            }
            for school_year in SchoolYear.objects.all()
        ]

    def queryset(self, request, queryset):
        return queryset.filter(**{self.field_path: self.school_year})



class ApprovedListFilter(admin.SimpleListFilter):
    title = _('approval')
    parameter_name = 'approved'

    def lookups(self, request, model_admin):
        return (
            ('yes', _('approved')),
            ('no', _('unapproved')),
        )

    def queryset(self, request, queryset):
        if self.value() == 'yes':
            return queryset.filter(approved__isnull=False)
        if self.value() == 'no':
            return queryset.filter(approved__isnull=True)



class CanceledListFilter(admin.SimpleListFilter):
    title = _('cancelation')
    parameter_name = 'canceled'

    def lookups(self, request, model_admin):
        return (
            (None, _('not canceled')),
            ('yes', _('canceled')),
        )

    def queryset(self, request, queryset):
        if self.value() == 'yes':
            return queryset.filter(canceled__isnull=False)
        else:
            return queryset.filter(canceled__isnull=True)

    def choices(self, cl):
        value = self.value()
        return [
            {
                'selected': value != 'yes',
                'query_string': cl.get_query_string({}, [self.parameter_name]),
                'display': _('active'),
            },
            {
                'selected': value == 'yes',
                'query_string': cl.get_query_string({self.parameter_name: 'yes'}),
                'display': _('canceled'),
            },
        ]



class SubjectTypeListFilter(admin.RelatedFieldListFilter):
    subject_type_type = None

    def __init__(self, field, request, params, model, model_admin, field_path):
        self.subject_types = SubjectType.objects.all()
        if self.subject_type_type:
            self.title = SubjectType.subject_type_type_labels[self.subject_type_type]
            self.subject_types = self.subject_types.filter(subject_type=self.subject_type_type)
        super(SubjectTypeListFilter, self).__init__(field, request, params, model, model_admin, field_path)
        if self.subject_type_type:
            self.title = SubjectType.subject_type_type_labels[self.subject_type_type]
        request.leprikon_subject_type_id = self.lookup_val

    def field_choices(self, field, request, model_admin):
        return [(t.id, t.plural) for t in self.subject_types.all()]


class CourseTypeListFilter(SubjectTypeListFilter):
    subject_type_type = SubjectType.COURSE


class EventTypeListFilter(SubjectTypeListFilter):
    subject_type_type = SubjectType.EVENT



class SubjectGroupListFilter(admin.RelatedFieldListFilter):
    subject_type_type = None

    def __init__(self, field, request, params, model, model_admin, field_path):
        if hasattr(request, 'leprikon_subject_type_id') and request.leprikon_subject_type_id:
            self.groups = SubjectGroup.objects.filter(subject_types__id=request.leprikon_subject_type_id)
        elif self.subject_type_type:
            self.groups = SubjectGroup.objects.filter(subject_types__subject_type=self.subject_type_type)
        else:
            self.groups = SubjectGroup.objects.all()
        super(SubjectGroupListFilter, self).__init__(field, request, params, model, model_admin, field_path)
        request.leprikon_subject_group_id = self.lookup_val

    def field_choices(self, field, request, model_admin):
        return [(g.id, g.plural) for g in self.groups.distinct()]


class CourseGroupListFilter(SubjectGroupListFilter):
    subject_type_type = SubjectType.COURSE


class EventGroupListFilter(SubjectGroupListFilter):
    subject_type_type = SubjectType.EVENT



class SubjectListFilter(admin.RelatedFieldListFilter):
    subject_type_type = None
    model = Subject

    def __init__(self, field, request, params, model, model_admin, field_path):
        self.subjects = self.model.objects.filter(school_year=request.school_year)
        if hasattr(request, 'leprikon_subject_type_id') and request.leprikon_subject_type_id:
            self.subjects = self.subjects.filter(subject_type__id=request.leprikon_subject_type_id)
        elif self.subject_type_type:
            self.subjects = self.subjects.filter(subject_type__subject_type=self.subject_type_type)
        if hasattr(request, 'leprikon_subject_group_id') and request.leprikon_subject_group_id:
            self.subjects = self.subjects.filter(subject_group__id=request.leprikon_subject_group_id)
        super(SubjectListFilter, self).__init__(field, request, params, model, model_admin, field_path)
        if self.subject_type_type:
            self.title = SubjectType.subject_type_labels[self.subject_type_type]

    def field_choices(self, field, request, model_admin):
        return [(s.id, s.name) for s in self.subjects]


class CourseListFilter(SubjectListFilter):
    subject_type_type = SubjectType.COURSE
    model = Course


class EventListFilter(SubjectListFilter):
    subject_type_type = SubjectType.EVENT
    model = Event



class LeaderListFilter(admin.RelatedFieldListFilter):
    def __init__(self, field, request, params, model, model_admin, field_path):
        self.leaders = Leader.objects.filter(school_years=request.school_year)
        super(LeaderListFilter, self).__init__(field, request, params, model, model_admin, field_path)

    def field_choices(self, field, request, model_admin):
        return [(leader.id, leader) for leader in self.leaders]



# -*- coding: utf-8 -*-
# Generated by Django 1.9.9 on 2016-09-06 15:36
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('leprikon', '0011_registration_requests'),
    ]

    operations = [
        migrations.AlterField(
            model_name='club',
            name='place',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='clubs', to='leprikon.Place', verbose_name='place'),
        ),
        migrations.AlterField(
            model_name='event',
            name='place',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='events', to='leprikon.Place', verbose_name='place'),
        ),
    ]



from __future__ import unicode_literals

from datetime import date

from django.db import models
from django.utils.encoding import python_2_unicode_compatible
from django.utils.functional import cached_property
from django.utils.translation import ugettext_lazy as _

from .startend import StartEndMixin


class SchoolYearManager(models.Manager):
    def get_current(self):
        # by default use last active year
        school_year = self.filter(active=True).order_by('-year').first()
        if school_year is None:
            # Create or activate current year
            if date.today().month < 7:
                year = date.today().year - 1
            else:
                year = date.today().year
            school_year = SchoolYear.objects.get_or_create(year=year)[0]
            school_year.active = True
            school_year.save()
        return school_year



@python_2_unicode_compatible
class SchoolYear(models.Model):
    year    = models.IntegerField(_('year'), unique=True)
    active  = models.BooleanField(_('active'), default=False)

    objects = SchoolYearManager()

    class Meta:
        app_label           = 'leprikon'
        ordering            = ('-year',)
        verbose_name        = _('school year')
        verbose_name_plural = _('school years')

    def __str__(self):
        return self.name

    def delete(self):
        if self.courses.count() or self.events.count():
            raise Exception(_('Can not delete sclool year with courses or events.'))
        super(SchoolYear, self).delete()

    @cached_property
    def name(self):
        return '{}/{}'.format(self.year, self.year + 1)



@python_2_unicode_compatible
class SchoolYearDivision(models.Model):
    school_year = models.ForeignKey(SchoolYear, verbose_name=_('school year'), related_name='divisions')
    name        = models.CharField(_('division name'), max_length=150)
    period_name = models.CharField(_('period name'), max_length=150)

    class Meta:
        app_label           = 'leprikon'
        ordering            = ('name',)
        unique_together     = (('school_year', 'name'),)
        verbose_name        = _('school year division')
        verbose_name_plural = _('school year divisions')

    def __str__(self):
        return self.name

    @cached_property
    def all_periods(self):
        return list(self.periods.all())

    def copy_to_school_year(old, school_year):
        new = SchoolYearDivision.objects.get(id=old.id)
        new.id, new.pk = None, None
        new.school_year = school_year
        new.save()
        year_offset = school_year.year - old.school_year.year
        periods = []
        for period in old.all_periods:
            try:
                start = date(period.start.year + year_offset, period.start.month, period.start.day)
            except ValueError:
                # handle leap-year
                start = date(period.start.year + year_offset, period.start.month, period.start.day - 1)
            try:
                end   = date(period.end.year   + year_offset, period.end.month,   period.end.day)
            except ValueError:
                # handle leap-year
                end   = date(period.end.year   + year_offset, period.end.month,   period.end.day - 1)
            periods.append(SchoolYearPeriod(
                school_year_division=new,
                name=period.name,
                start=start,
                end=end,
            ))
        SchoolYearPeriod.objects.bulk_create(periods)
        return new



@python_2_unicode_compatible
class SchoolYearPeriod(StartEndMixin, models.Model):
    school_year_division = models.ForeignKey(SchoolYearDivision, verbose_name=_('school year division'),
                                             related_name='periods')
    name        = models.CharField(_('name'), max_length=150)
    start       = models.DateField(_('start date'))
    end         = models.DateField(_('end date'))

    class Meta:
        app_label           = 'leprikon'
        ordering            = ('start',)
        verbose_name        = _('school year period')
        verbose_name_plural = _('school year periods')

    def __str__(self):
        return _('{name}, {start:%m/%d %y} - {end:%m/%d %y}').format(
            name    = self.name,
            start   = self.start,
            end     = self.end,
        )



from __future__ import unicode_literals

from json import loads

from django.core.exceptions import ValidationError
from django.db import models
from django.utils.encoding import python_2_unicode_compatible
from django.utils.functional import cached_property
from django.utils.module_loading import import_string
from django.utils.translation import ugettext_lazy as _

from ..conf import settings
from ..utils import first_upper


@python_2_unicode_compatible
class Question(models.Model):
    name        = models.CharField(_('name'), max_length=50, unique=True)
    question    = models.CharField(_('question'), max_length=50)
    help_text   = models.TextField(_('help text'), blank=True, null=True,
                                   help_text=_('This is help text. The help text is shown next to the form field.'))
    field       = models.CharField(_('field'), max_length=150, choices=(
        (key, val['name']) for key, val in settings.LEPRIKON_QUESTION_FIELDS.items()
    ))
    field_args  = models.TextField(_('field_args'), blank=True, default='{}',
                                   help_text=_('Enter valid JSON structure representing field configuration.'))

    class Meta:
        app_label           = 'leprikon'
        verbose_name        = _('additional question')
        verbose_name_plural = _('additional questions')

    def __str__(self):
        return self.question

    @cached_property
    def field_class(self):
        return import_string(settings.LEPRIKON_QUESTION_FIELDS[self.field]['class'])

    @cached_property
    def field_kwargs(self):
        return loads(self.field_args)

    @cached_property
    def field_label(self):
        return first_upper(self.question)

    def get_field(self, initial=None):
        return self.field_class(
            label       = self.field_label,
            initial     = initial,
            help_text   = self.help_text,
            **self.field_kwargs
        )

    def clean(self):
        try:
            self.get_field()
        except Exception as e:
            raise ValidationError({'field_args': [_('Failed to create field with given field args: {}').format(e)]})



# -*- coding: utf-8 -*-
# Generated by Django 1.9.5 on 2016-05-24 16:02
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion
import filer.fields.file


class Migration(migrations.Migration):

    dependencies = [
        ('filer', '0001_initial'),
        ('leprikon', '0004_event_list_plugin'),
    ]

    operations = [
        migrations.CreateModel(
            name='MessageAttachment',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('file', filer.fields.file.FilerFileField(on_delete=django.db.models.deletion.CASCADE, related_name='+', to='filer.File')),
                ('message', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='attachments', to='leprikon.Message', verbose_name='message')),
            ],
            options={
                'verbose_name': 'attachment',
                'verbose_name_plural': 'attachments',
            },
        ),
    ]



from __future__ import unicode_literals

from collections import namedtuple

from . import settings


class VerifiedEmailFieldSetup(namedtuple('VerifiedEmailFieldSetup', (
        'cache_prefix', 'code_length', 'code_ttl', 'mail_from', 'mail_subject',
        'mail_template_txt', 'mail_template_html', 'mail_context', 'mail_mailer'))):
    def __new__(
            cls, cache_prefix=settings.CACHE_PREFIX,
            code_length=settings.CODE_LENGTH, code_ttl=settings.CODE_TTL,
            mail_from=settings.MAIL_FROM, mail_subject=settings.MAIL_SUBJECT,
            mail_template_txt=settings.MAIL_TEMPLATE_TXT, mail_template_html=settings.MAIL_TEMPLATE_HTML,
            mail_context=None, mail_mailer=settings.MAIL_MAILER, **kwargs):
        return super(VerifiedEmailFieldSetup, cls).__new__(
            cls, cache_prefix=cache_prefix,
            code_length=code_length, code_ttl=code_ttl,
            mail_from=mail_from, mail_subject=mail_subject,
            mail_template_txt=mail_template_txt, mail_template_html=mail_template_html,
            mail_context=mail_context or {}, mail_mailer=mail_mailer,
        )


fieldsetups = {}



from __future__ import unicode_literals

from django.http import (
    HttpResponse, HttpResponseBadRequest, HttpResponseNotFound,
)
from django.views.decorators.http import require_POST

from .fieldsetup import fieldsetups
from .forms import SendForm
from .utils import send_code


@require_POST
def send(request, fieldsetup_id):
    try:
        fieldsetup = fieldsetups[fieldsetup_id]
    except KeyError:
        return HttpResponseNotFound()
    form = SendForm(data=request.POST)
    if not form.is_valid():
        return HttpResponseBadRequest()
    send_code(form.cleaned_data['email'], fieldsetup)
    return HttpResponse()



from __future__ import unicode_literals

from django.conf import settings
from django.utils.translation import ugettext_lazy as _

CACHE_PREFIX = getattr(settings, 'VERIFIED_EMAIL_CACHE_PREFIX', 'verified_email_field_')
CODE_LENGTH = int(getattr(settings, 'VERIFIED_EMAIL_CODE_LENGTH', 6))
CODE_TTL = int(getattr(settings, 'VERIFIED_EMAIL_CODE_TTL', 300))
MAIL_FROM = getattr(settings, 'VERIFIED_EMAIL_MAIL_FROM', settings.SERVER_EMAIL)
MAIL_SUBJECT = getattr(settings, 'VERIFIED_EMAIL_MAIL_SUBJECT', _('Verification code'))
MAIL_TEMPLATE_TXT = getattr(settings, 'VERIFIED_EMAIL_MAIL_TEMPLATE_TXT', 'verified_email_field/mail.txt')
MAIL_TEMPLATE_HTML = getattr(settings, 'VERIFIED_EMAIL_MAIL_TEMPLATE_TXT', 'verified_email_field/mail.html')
MAIL_MAILER = getattr(settings, 'VERIFIED_EMAIL_MAIL_MAILER',
                      'Django Verified Email Field (https://github.com/misli/django-verified-email-field)')



from __future__ import unicode_literals

from django import forms
from django.contrib import admin
from django.contrib.auth import get_user_model
from django.core.urlresolvers import reverse
from django.shortcuts import render
from django.utils.functional import cached_property
from django.utils.translation import ugettext_lazy as _

from ..models.roles import Contact
from ..models.schoolyear import SchoolYear
from ..models.subjects import SubjectType
from .filters import SchoolYearListFilter
from .messages import SendMessageAdminMixin


class ContactInlineAdmin(admin.TabularInline):
    model = Contact
    extra = 0



class LeaderAdmin(SendMessageAdminMixin, admin.ModelAdmin):
    filter_horizontal   = ('school_years',)
    inlines             = (ContactInlineAdmin,)
    search_fields       = ('user__first_name', 'user__last_name', 'contacts__contact')
    list_display        = ('id', 'first_name', 'last_name', 'email', 'courses_link',
                           'events_link', 'contacts', 'user_link', 'icon')
    ordering            = ('user__first_name', 'user__last_name')
    actions             = ('add_school_year',)
    list_filter         = (('school_years', SchoolYearListFilter),)
    raw_id_fields       = ('user',)

    def add_school_year(self, request, queryset):

        class SchoolYearForm(forms.Form):
            school_year = forms.ModelChoiceField(
                label=_('Target school year'),
                help_text=_('All selected leaders will be added to selected school year.'),
                queryset=SchoolYear.objects.all(),
            )
        if request.POST.get('post', 'no') == 'yes':
            form = SchoolYearForm(request.POST)
            if form.is_valid():
                school_year = form.cleaned_data['school_year']
                for leader in queryset.all():
                    leader.school_years.add(school_year)
                self.message_user(request, _('Selected leaders were added to school year {}.').format(school_year))
                return
        else:
            form = SchoolYearForm()
        return render(request, 'leprikon/admin/action_form.html', {
            'title': _('Select target school year'),
            'queryset': queryset,
            'opts': self.model._meta,
            'form': form,
            'action': 'add_school_year',
            'action_checkbox_name': admin.helpers.ACTION_CHECKBOX_NAME,
        })
    add_school_year.short_description = _('Add selected leaders to another school year')

    def first_name(self, obj):
        return obj.user.first_name
    first_name.short_description = _('first name')
    first_name.admin_order_field = 'user__first_name'

    def last_name(self, obj):
        return obj.user.last_name
    last_name.short_description = _('last name')
    last_name.admin_order_field = 'user__last_name'

    def email(self, obj):
        return obj.user.email
    email.short_description = _('email')
    email.admin_order_field = 'user__email'

    def contacts(self, obj):
        return ', '.join(c.contact for c in obj.all_contacts)
    contacts.short_description = _('contacts')

    def user_link(self, obj):
        return '<a href="{url}">{user}</a>'.format(
            url     = reverse('admin:auth_user_change', args=(obj.user.id,)),
            user    = obj.user,
        )
    user_link.allow_tags = True
    user_link.short_description = _('user')

    @cached_property
    def courses_url(self):
        return reverse('admin:leprikon_course_changelist')

    @cached_property
    def events_url(self):
        return reverse('admin:leprikon_event_changelist')

    def courses_link(self, obj):
        return '<a href="{url}?leaders__id={leader}">{count}</a>'.format(
            url     = self.courses_url,
            leader  = obj.id,
            count   = obj.subjects.filter(subject_type__subject_type=SubjectType.COURSE).count(),
        )
    courses_link.allow_tags = True
    courses_link.short_description = _('courses')

    def events_link(self, obj):
        return '<a href="{url}?leaders__id={leader}">{count}</a>'.format(
            url     = self.events_url,
            leader  = obj.id,
            count   = obj.subjects.filter(subject_type__subject_type=SubjectType.EVENT).count(),
        )
    events_link.allow_tags = True
    events_link.short_description = _('events')

    def icon(self, obj):
        try:
            return '<img src="{}" alt="{}"/>'.format(obj.photo.icons['48'], obj.photo.label)
        except (AttributeError, KeyError):
            return ''
    icon.allow_tags = True
    icon.short_description = _('photo')

    def get_message_recipients(self, request, queryset):
        return get_user_model().objects.filter(
            leprikon_leader__in = queryset
        ).distinct()



class ParentAdmin(SendMessageAdminMixin, admin.ModelAdmin):
    search_fields   = ('first_name', 'last_name', 'street', 'email', 'phone',
                       'user__first_name', 'user__last_name', 'user__username', 'user__email')
    list_display    = ('id', 'first_name', 'last_name', 'address', 'email', 'phone', 'user_link')
    raw_id_fields   = ('user',)

    def first_name(self, obj):
        return obj.user.first_name
    first_name.short_description = _('first name')
    first_name.admin_order_field = 'user__first_name'

    def last_name(self, obj):
        return obj.user.last_name
    last_name.short_description = _('last name')
    last_name.admin_order_field = 'user__last_name'

    def address(self, obj):
        return obj.address
    address.short_description = _('address')

    def email(self, obj):
        return obj.user.email
    email.short_description = _('email')
    email.admin_order_field = 'user__email'

    @cached_property
    def users_url(self):
        return reverse('admin:auth_user_changelist')

    def user_link(self, obj):
        return '<a href="{url}?id={id}">{user}</a>'.format(
            url     = self.users_url,
            id      = obj.user.id,
            user    = obj.user,
        )
    user_link.allow_tags = True
    user_link.short_description = _('user')

    def get_message_recipients(self, request, queryset):
        return get_user_model().objects.filter(
            leprikon_parents__in = queryset
        ).distinct()



class ParticipantAdmin(SendMessageAdminMixin, admin.ModelAdmin):
    search_fields   = ('first_name', 'last_name', 'birth_num', 'street', 'email', 'phone',
                       'user__first_name', 'user__last_name', 'user__username', 'user__email')
    list_display    = ('id', 'first_name', 'last_name', 'birth_num', 'gender',
                       'address', 'email', 'phone', 'school_name',
                       'user_link', 'registrations_links')
    raw_id_fields   = ('user',)

    @cached_property
    def users_url(self):
        return reverse('admin:auth_user_changelist')

    def user_link(self, obj):
        return '<a href="{url}?id={id}">{user}</a>'.format(
            url     = self.users_url,
            id      = obj.user.id,
            user    = obj.user,
        )
    user_link.allow_tags = True
    user_link.short_description = _('user')

    def address(self, obj):
        return obj.address
    address.short_description = _('address')

    @cached_property
    def course_regs_url(self):
        return reverse('admin:leprikon_courseregistration_changelist')

    @cached_property
    def event_regs_url(self):
        return reverse('admin:leprikon_eventregistration_changelist')

    def registrations_links(self, obj):
        return (
            '<a href="{course_regs_url}?participant_birth_num={birth_num}">{course_regs_name}</a>, '
            '<a href="{event_regs_url}?participant_birth_num={birth_num}">{event_regs_name}</a>'.format(
                course_regs_url = self.course_regs_url,
                event_regs_url  = self.event_regs_url,
                course_regs_name= _('courses'),
                event_regs_name = _('events'),
                birth_num       = obj.birth_num,
            )
        )
    registrations_links.allow_tags = True
    registrations_links.short_description = _('registrations')

    def school_name(self, obj):
        return obj.school_name
    school_name.short_description = _('school')

    def get_message_recipients(self, request, queryset):
        return get_user_model().objects.filter(
            leprikon_participants__in = queryset
        ).distinct()



from __future__ import unicode_literals

from django import forms
from django.core.urlresolvers import reverse
from django.forms.widgets import EmailInput, MultiWidget, TextInput
from django.templatetags.static import static
from django.utils.html import format_html


class VerifiedEmailWidget(MultiWidget):
    def __init__(self, send_label, fieldsetup_id, email_attrs, code_attrs):
        self.send_label = send_label
        self.fieldsetup_id = fieldsetup_id
        widgets = (EmailInput(attrs=email_attrs), TextInput(attrs=code_attrs))
        super(VerifiedEmailWidget, self).__init__(widgets)

    @property
    def media(self):
        print('##################', static('verified_email_field/send.js'))
        raise Exception(static('verified_email_field/send.js'))
        return forms.Media(js=[static('verified_email_field/send.js')])

    def decompress(self, value):
        if value:
            return [value, None]
        return [None, None]

    def render(self, name, value, attrs=None):
        self.name = name
        if not attrs:
            attrs = {}
        attrs['class'] = attrs['class'] + ' form-control' if 'class' in attrs else 'form-control'
        return super(VerifiedEmailWidget, self).render(name, value, attrs)
    
    def get_context(self, name, value, attrs):
        context = super(VerifiedEmailWidget, self).get_context(name, value, attrs)
        context['name'] = self.name
        context['fieldsetup_id'] = self.fieldsetup_id
        context['send_label'] = self.send_label
        return context

    # format_output is removed in Django 1.11
    # It is only retained here for Django 1.8 compatibility
    def format_output(self, rendered_widgets):
        return ''.join((
            rendered_widgets[0],
            '<div class="input-group">',
            '<span class="input-group-btn">',
            format_html(
                '<button class="btn btn-default" type="button" '
                'onclick="send_verification_code(\'{}\', \'{}\')">{}</button>',
                self.name,
                reverse('verified-email-field:send', args=(self.fieldsetup_id,)),
                self.send_label,
            ),
            '</span>',
            rendered_widgets[1],
            '</div>',
        ))



from __future__ import unicode_literals

from django.db import models
from django.utils.encoding import python_2_unicode_compatible
from django.utils.translation import ugettext_lazy as _


@python_2_unicode_compatible
class Department(models.Model):
    name    = models.CharField(_('name'), max_length=50)
    code    = models.PositiveSmallIntegerField(_('department code'), blank=True, default=0)

    class Meta:
        app_label           = 'leprikon'
        ordering            = ('code', 'name')
        verbose_name        = _('department')
        verbose_name_plural = _('departments')

    def __str__(self):
        if self.code:
            return '{code} {name}'.format(code=self.code, name=self.name)
        else:
            return '{name}'.format(name=self.name)



from __future__ import unicode_literals

from django import template
from django.conf import settings

register = template.Library()


@register.inclusion_tag('rocketchat/livechat.html')
def livechat():
    return {
        'rocketchat_url': getattr(settings, 'ROCKETCHAT_URL', None),
        'DEBUG': settings.DEBUG,
    }
